@@ -2,7 +2,7 @@
  *
  * SPDX-License-Identifier: BSD-2-Clause
  *
- * Copyright (c) 2024 Steffen Nurpmeso <steffen@sdaoden.eu>.
+ * Copyright (c) 2024 - 2025 Steffen Nurpmeso <steffen@sdaoden.eu>.
  * (Only technical surroundings, algorithm is solely Colin Percival.)
  *
  * Copyright 2003-2005 Colin Percival
@@ -110,7 +110,7 @@ s_bsdipa_patch_parse_header(struct s_bsdipa_header *hp, uint8_t const *dat){
 	x = a_bspatch_xin(dat);
 	if(x < 0)
 		goto jleave;
-	y = x;
+	y = x; /* If we generated the header, data *plus* control block fits in _OFF_MAX! */
 	hp->h_ctrl_len = x;
 	dat += sizeof(x);
 
@@ -134,6 +134,7 @@ s_bsdipa_patch_parse_header(struct s_bsdipa_header *hp, uint8_t const *dat){
 	x = a_bspatch_xin(dat);
 	if(x < 0)
 		goto jleave;
+	/* If we generated the header, the latter is true. */
 	if(x >= s_BSDIPA_OFF_MAX || (size_t)x >= SIZE_MAX / sizeof(s_bsdipa_off_t))
 		goto jleave;
 	if(x - hp->h_extra_len != hp->h_diff_len)
@@ -163,38 +164,58 @@ s_bsdipa_patch(struct s_bsdipa_patch_ctx *pcp){
 	rv = s_BSDIPA_INVAL;
 
 	if(pcp->pc_patch_dat != NULL){
-		s_bsdipa_off_t y;
 		uint64_t x;
 
 		pcp->pc_diff_dat = pcp->pc_ctrl_dat = pcp->pc_patch_dat;
 
 		x = pcp->pc_patch_len;
 
-		y = pcp->pc_header.h_ctrl_len;
-		if(y < 0 || x < (uint64_t)y)
+		respos = pcp->pc_header.h_ctrl_len;
+		if(respos < 0 || x < (uint64_t)respos)
 			goto jleave;
-		x -= y;
-		pcp->pc_diff_dat += y;
+		x -= respos;
+		pcp->pc_diff_dat += respos;
 
-		y = pcp->pc_header.h_diff_len;
-		if(y < 0 || x < (uint64_t)y)
+		respos = pcp->pc_header.h_diff_len;
+		if(respos < 0 || x < (uint64_t)respos)
 			goto jleave;
-		x -= y;
-		pcp->pc_diff_dat += y;
+		x -= respos;
+		pcp->pc_diff_dat += respos;
 
 		pcp->pc_extra_dat = pcp->pc_diff_dat;
-		y = pcp->pc_header.h_extra_len;
-		if(y < 0 || x < (uint64_t)y)
+		respos = pcp->pc_header.h_extra_len;
+		if(respos < 0 || x < (uint64_t)respos)
 			goto jleave;
 		/* xxx Do not care about excess data in patch? */
+	}else{
+		respos = pcp->pc_header.h_ctrl_len;
+		if(respos < 0 || respos >= s_BSDIPA_OFF_MAX)
+			goto jleave;
+		respos = pcp->pc_header.h_diff_len;
+		if(respos < 0 || respos >= s_BSDIPA_OFF_MAX)
+			goto jleave;
+		aftpos = pcp->pc_header.h_extra_len;
+		if(aftpos < 0 || aftpos >= s_BSDIPA_OFF_MAX)
+			goto jleave;
+		if(s_BSDIPA_OFF_MAX - aftpos <= respos)
+			goto jleave;
+		respos += aftpos;
+		if(s_BSDIPA_OFF_MAX - respos <= pcp->pc_header.h_ctrl_len)
+			goto jleave;
 	}
 
 	/* The effective limit is smaller, but that is up to diff generation: "just do it" */
 	if(pcp->pc_after_len >= s_BSDIPA_OFF_MAX)
 		goto jleave;
-	if(pcp->pc_header.h_before_len >= s_BSDIPA_OFF_MAX)
+
+	if(pcp->pc_header.h_before_len < 0 || pcp->pc_header.h_before_len >= s_BSDIPA_OFF_MAX)
 		goto jleave;
 
+	if(pcp->pc_max_allowed_restored_len != 0 &&
+			pcp->pc_max_allowed_restored_len < (uint64_t)pcp->pc_header.h_before_len){
+		rv = s_BSDIPA_FBIG;
+		goto jleave;
+	}
 	pcp->pc_restored_len = pcp->pc_header.h_before_len;
 
 	/* Ensure room for one additional byte, as documented. */
